//==============================================================================
// http://narodstream.ru/avr-urok-34-displej-tft-240x320-8bit-chast-1/
//==============================================================================
#include "main.h"
#include <stdlib.h>    // апределение abs()
//
#include <avr/eeprom.h>

unsigned char EEMEM ufont8x5[][5] =      // это в пзу
{
    {0x3E,0x51,0x49,0x45,0x3E}, // 0
    {0x00,0x42,0x7F,0x40,0x00}, // 1
    {0x42,0x61,0x51,0x49,0x46}, // 2
    {0x21,0x41,0x45,0x4B,0x31}, // 3
    {0x18,0x14,0x12,0x7F,0x10}, // 4
    {0x27,0x45,0x45,0x45,0x39}, // 5
    {0x3C,0x4A,0x49,0x49,0x30}, // 6
    {0x01,0x71,0x09,0x05,0x03}, // 7
    {0x36,0x49,0x49,0x49,0x36}, // 8
    {0x06,0x49,0x49,0x29,0x1E}, // 9
    {0x00,0x36,0x36,0x00,0x00}, // :

    {0x46,0x49,0x49,0x49,0x31}, // 11 'S'
    {0x1C,0x20,0x40,0x20,0x1C}, // 12 'v' 
    {0x01,0x01,0x7F,0x01,0x01}, // 13 'T'
    {0x14,0x14,0x14,0x14,0x14}, // 14 '='
    {0x00,0x06,0x09,0x09,0x06}, // 15 "кружок для градусов
//    {0x7F,0x40,0x40,0x40,0xFF}, // 16 'Ц'
    {0x3E,0x41,0x41,0x41,0x22}, // 16 'C'
    {0x00,0x50,0x30,0x00,0x00}, // 17 ',' запятая
    {0x7F,0x49,0x49,0x49,0x36}, // 18 'В'
    {0x20,0x41,0x3F,0x01,0x7F}, // 19 'Л'
//  {0x20,0x44,0x3C,0x04,0x7C}, // 19 'л'
    {0x23,0x13,0x08,0x64,0x62}  // 20 '%'
};

unsigned int X_SIZE = 320;
unsigned int Y_SIZE = 240;

#define PORT_DATA   PORTD // порт данных
#define DDR_DATA    DDRD

#define DATA_DDR    DDRD
#define DATA_PORT   PORTD
#define DATA_PIN    PIND
//-------------------------------- порт управления
#define COMMAND_DDR   DDRC
#define COMMAND_PORT  PORTC

#define LCD_RESET   3
#define LCD_CD      2
#define LCD_RS      2
#define LCD_WR      1
#define LCD_RD      0

#define WR_IDLE     COMMAND_PORT |= (1<<LCD_WR)
#define RD_IDLE     COMMAND_PORT |= (1<<LCD_RD)
#define WR_ACTIVE   COMMAND_PORT &= ~(1<<LCD_WR)
#define RD_ACTIVE   COMMAND_PORT &= ~(1<<LCD_RD)
#define CD_COMMAND  COMMAND_PORT &= ~(1<<LCD_CD)
#define CD_DATA     COMMAND_PORT |= (1<<LCD_CD)

#define setReadDir()  DATA_DDR = 0x00
#define setWriteDir()   DATA_DDR = 0xFF
#define WR_STROBE     {WR_ACTIVE;WR_IDLE;}

//==============================================================================
void port_ini(void)
{ DATA_PORT   = 0x00;
  DATA_DDR    = 0xFF;     //Шина данных на выход
  COMMAND_DDR |= (1<<LCD_RESET)|(1<<LCD_CD)|(1<<LCD_WR)|(1<<LCD_RD);
}
//==============================================================================
void TFT9341_Write8(unsigned char data)
{ DATA_PORT = data;
  WR_STROBE;
}
//==============================================================================
void TFT9341_SendCommand(unsigned char cmd)
{ CD_COMMAND;   // лапка в состоянии посылки команды
  TFT9341_Write8(cmd);  
}
//==============================================================================
void TFT9341_SendData(unsigned char data)
{ CD_DATA;    // лапка в состоянии посылки данных
  TFT9341_Write8(data);  
}
//==============================================================================
void TFT9341_reset(void)
{ port_ini();
  WR_IDLE;
  RD_IDLE;
  COMMAND_PORT &= ~(1<<LCD_RESET);
  _delay_ms(2);
  COMMAND_PORT |= (1<<LCD_RESET);
  TFT9341_SendCommand(0x01); // Software Reset
//  for (uint8_t i=0;i<3;i++) WR_STROBE;
}
//==============================================================================
void TFT9341_WriteRegister32(unsigned char r, unsigned long d)
{ CD_COMMAND;
    TFT9341_Write8(r);
    CD_DATA;
    _delay_us(1);
    TFT9341_Write8(d>>24);
    _delay_us(1);
    TFT9341_Write8(d>>16);
    _delay_us(1);
    TFT9341_Write8(d>>8);
    _delay_us(1);
    TFT9341_Write8(d);
}
//==============================================================================
void TFT9341_SetAddrWindow(
  unsigned int x1,unsigned int y1,
  unsigned int x2,unsigned int y2)
{ unsigned long t;
    t = x1;
    t<<=16;
    t |= x2;
    TFT9341_WriteRegister32(0x2A,t);//Column Addres Set
    t = y1;
    t<<=16;
    t |= y2;
    TFT9341_WriteRegister32(0x2B,t);//Page Addres Set
}
//==============================================================================
void TFT9341_DrawPixel(
  int x, // по вертикали
  int y, // по горизонтали
  unsigned int color)
{
// сначала проверка на валидность
  if((x<0)||(y<0)||(x>=X_SIZE)||(y>=Y_SIZE)) return;
 ////   
 TFT9341_SetAddrWindow(x,y,X_SIZE-1,Y_SIZE-1);
//    
  CD_COMMAND;   
  TFT9341_Write8(0x2C);
  CD_DATA;
  TFT9341_Write8(color>>8); TFT9341_Write8(color);
}
//==============================================================================
void SVLine( // рисуем вентикальную линию
  unsigned int x1,                  // по горизонтали
  unsigned int y1, unsigned int y2, // низ-верх
  unsigned int color)
{
  unsigned int t = y1;
  while(t < y2)
  {
    TFT9341_DrawPixel(t, x1, color);
    t++;
  } 
}
//==============================================================================
void SHLine( // рисуем горизонтальную линию
  unsigned int x1,unsigned int x2, // лево-право
  unsigned int y1, // по вертикали
  unsigned int color)
{
  unsigned int t=x1;
  while (t < x2)
  {
    TFT9341_DrawPixel(y1,t,color);
    t++;
  } 
}
//==============================================================================
// вариант 1 будет у нас для шрифта размером 5x8 пикселей
void put_char(
  unsigned int x,     // по вертикали
  unsigned int y,     // по горизонтали
  unsigned char size,
  unsigned int color, unsigned int backcolor, 
  unsigned char ch)
{
  unsigned char value = 0;
  unsigned char mask[8] =
  { 0b10000000,
    0b01000000,
    0b00100000,
    0b00010000,
    0b00001000,
    0b00000100,
    0b00000010,
    0b00000001
  };

#define mnVALUE value = eeprom_read_byte(&ufont8x5[ch][h]);
  switch(size)
    {
      int v,h;
      int tv,th;
      int tcolor;

    case 1:
        for(h = 0; h < 5; h++)        // по горизонтали (y)
        { 
        mnVALUE
        
        for(v = 0; v < 8; v++)      // по вертикали (x)
        {
          if ((value & mask[v]) != 0) tcolor = color;
          else tcolor = backcolor;

          TFT9341_DrawPixel(x+v,y+h,tcolor);
        }
        }
//В принципе, несложно догадаться, что мы из элемента массива собираем пиксели по горизонтали (i) и по вертикали (h).
        break;
 
      case 2:
        for(h = 0; h < 5; h++)        // по горизонтали (y)
        { 
        mnVALUE

        th = 2*h;

          for(v = 0; v < 8; v++)      // по вертикали (x)
        { 
          tv = 2*v;
          
          if ((value & mask[v]) != 0) tcolor = color;
          else tcolor = backcolor;

          TFT9341_DrawPixel(x + tv,   y + th,   tcolor);
          TFT9341_DrawPixel(x + tv,   y + (th+1), tcolor);
          TFT9341_DrawPixel(x + (tv+1), y + th,   tcolor);
          TFT9341_DrawPixel(x + (tv+1), y + (th+1), tcolor);
          }
      }
      break;
//------------------------------------------------------------------------------
    case 3:
        for(h = 0; h < 5; h++)        // по горизонтали (y)
        { 
        mnVALUE

        th = 3*h;

        for(v = 0; v < 8; v++)      // по вертикали (x)
        { 
          tv = 3*v;
          
          if ((value & mask[v]) != 0) tcolor = color;
          else tcolor = backcolor;

          TFT9341_DrawPixel(x + tv,   y + th,   tcolor);
          TFT9341_DrawPixel(x + tv,   y + (th+1), tcolor);
          TFT9341_DrawPixel(x + tv,   y + (th+2), tcolor);
          TFT9341_DrawPixel(x + (tv+1), y + th,   tcolor);
          TFT9341_DrawPixel(x + (tv+1), y + (th+1), tcolor);
          TFT9341_DrawPixel(x + (tv+1), y + (th+2), tcolor);
          TFT9341_DrawPixel(x + (tv+2), y + th,   tcolor);
          TFT9341_DrawPixel(x + (tv+2), y + (th+1), tcolor);
          TFT9341_DrawPixel(x + (tv+2), y + (th+2), tcolor);
          }
      }
      break;
    }
}
//==============================================================================

